<?php

namespace FiloBlu\OrderFlowRectifier\Model\Rectifiers;

class EsbScheduleUpdateRectifier implements \FiloBlu\OrderFlowRectifier\Api\RectifierInterface
{
    const PREV_MINUTES = 30;
    const ORDER_EXPORTED = "order_exported";
    const PAYMENT_EXPORTED = "payment_exported";
    const CREATE_ORDER = "create_order";
    const FULFILL_ORDER = "fulfill_order";

    protected $connection;

    public function __construct(
        \Magento\Framework\App\ResourceConnection $resource)
    {
        $this->connection = $resource->getConnection();
    }

    public function getInfo() {
        return 'Manage data on ESB for fix change of status problem.';
    }

    public function getTitle() {
        return 'Insert data on ESB';
    }

    /**
     * @param $order
     * @return bool
     */
    public function executeSingle($order) {
        $esbScheduleTable = $this->connection->getTableName('esb_schedule');
        $affectedRows = $this->connection->insert(
            $esbScheduleTable,
            [
                'object_class' => null,
                'object_repository_interface' => "\\Magento\\Sales\\Api\\OrderRepositoryInterface",
                'object_id' => $order->entity_id,
                'event' => "forced_from_script",
                'sendmessage_class' => "\\Filoblu\\Esb\\Cron\\SendMessage",
                'object_after_method' => "updateOrderStatus",
                'object_after_method_argument' => $order->object_after_method_argument,
                'endpoint' => "http://talend.filoblu.com:8088/rest/in/magento/".$order->endpoint."/",
                'status' => "pending",
                'response' => "",
                'priority' => $order->priority,
                'published_at' => $order->date,
                'executable_after' => $order->date,
                'executed_at' => null,
                'finished_at' => null
            ]
        );

        return $affectedRows > 0;
    }

    public function loadOrders() {
        $orderTable = $this->connection->getTableName('sales_order');
        $invoiceTable = $this->connection->getTableName('sales_invoice');
        $now = new \DateTime();
        $limit = $now->sub(new \DateInterval('PT'.self::PREV_MINUTES.'M'));

        $query = "SELECT so.entity_id, so.increment_id, 1 as priority, 
                 '".self::ORDER_EXPORTED."' as object_after_method_argument,
                 '".self::CREATE_ORDER."' as endpoint,
                 '".$now->format('Y-m-d H:i:s')."' as date
FROM {$orderTable} so
WHERE so.`status` IN ('confirmed', 'order_exported', 'payment_exported', 'payment_confirmed') 
AND so.`updated_at` < '".$limit->format('Y-m-d H:i:s')."'
ORDER BY so.`created_at`";

        $ordersToBeRectified = $this->connection->fetchAll($query, [], \Zend_Db::FETCH_OBJ);

        $entityIdToBeRectified = array();
        if(!empty($ordersToBeRectified))
        {
            foreach ($ordersToBeRectified as $order)
            {
                $entityIdToBeRectified[] = $order->entity_id;
            }
        }

        $query2 = "SELECT so.entity_id, so.increment_id, 2 as priority, 
                  '".self::PAYMENT_EXPORTED."' as object_after_method_argument,
                  '".self::FULFILL_ORDER."' as endpoint,
                  '".$now->format('Y-m-d H:i:s')."' as date
FROM {$orderTable} so
INNER JOIN {$invoiceTable} si
ON so.entity_id = si.order_id
WHERE so.`status` IN ('payment_exported', 'payment_confirmed') 
AND so.`updated_at` < '".$limit->format('Y-m-d H:i:s')."'
ORDER BY so.`created_at`";

        $ordersToBeRectified2 = $this->connection->fetchAll($query2, [], \Zend_Db::FETCH_OBJ);

        if(!empty($ordersToBeRectified2))
        {
            foreach ($ordersToBeRectified2 as $value)
            {
                if(!in_array($value->entity_id, $entityIdToBeRectified))
                {
                    $ordersToBeRectified[] = $value;
                }
            }
        }

        return $ordersToBeRectified;
    }
}